==> utensor_cgen-0.1.4.tar.gz: utensor_cgen-0.1.4/utensor_cgen/code_generator.py <==
# -*- coding:utf8 -*-
import os

import idx2numpy as idx2np
import numpy as np
import tensorflow as tf

from .operators import OperatorFactory
from .optimizer import Optimizer
from .parser import parse_pb
from .snippets import (CommentSnippet, ContextHeaderSnippet,
                       ContextSnippetsContainer, CreateTensorIdxSnippet)
from .snippets.composer import Composer

__all__ = ["CodeGenerator"]


class CodeGenerator(object):
  def __init__(self, pb_file, idx_dir, embed_data_dir, method, debug_cmt=False, output_nodes=None):
    self.pb_file = pb_file
    if not os.path.exists(idx_dir):
      os.makedirs(idx_dir)
    self.idx_dir = idx_dir
    self.embed_data_dir = embed_data_dir.rstrip("/")
    self.method = method
    self.debug_cmt = debug_cmt
    self.output_nodes = output_nodes

  def generate(self, src_fname):
    """Generate source and header files
    """
    fname, _ = os.path.splitext(src_fname)
    graph_name, _ = os.path.splitext(os.path.basename(self.pb_file))
    guard_name = fname.replace('/', '_')
    header_snippet = ContextHeaderSnippet(guard_name, graph_name)

    composer = Composer()
    header_fname = '{}.hpp'.format(fname)
    header_name = os.path.basename(header_fname)
    container = ContextSnippetsContainer(graph_name, header_name)

    opFactory = OperatorFactory()

    print("Parsing {}".format(self.pb_file))
    ops_info, ops_torder, output_nodes = parse_pb(self.pb_file, self.output_nodes)
    construct_order = Optimizer.optimize(ops_info, ops_torder, output_nodes, self.method)

    # TODO better snippet construction abstraction
    for op_id, (op_name, op_info, ref_counts, to_eval) in enumerate(construct_order, 1):
      op_type = op_info.op_type
      if op_type == "Placeholder":
        out_tname, _, _ = op_info.output_tensor[0]
        ref_count = ref_counts[0]
        container.template_vars["placeholders"].append(out_tname)
        container.template_vars["ref_counts"].append(ref_count)
        header_snippet.template_vars["placeholders"].append(out_tname)
      elif op_type == 'Const':
        out_tname, out_dtype, _ = op_info.output_tensor[0]
        ref_count = ref_counts[0]
        pre_tname = self._prepare_tensor_name(out_tname)
        idx_fname = "{}.idx".format(pre_tname)
        snippet = CreateTensorIdxSnippet(self.embed_data_dir, out_tname,
                                         idx_fname=idx_fname,
                                         tf_dtype=out_dtype,
                                         ref_count=ref_count)
        container.add_snippet(snippet)
        idx_path = os.path.join(self.idx_dir, idx_fname)
        value = op_info.output_content[out_tname]
        self._save_data(idx_path, value, out_dtype)
      else:
        snippet = opFactory.createOperatorSnippet(op_info, ref_counts, to_eval)
        container.add_snippet(snippet)

      if self.debug_cmt:
        comments = ["<<< Operation id {}: {}".format(op_id, op_name),
                    ">>> Operation id {}: {}".format(op_id+1, op_name)]
        cmt_snippet = CommentSnippet(comments)
        container.add_snippet(cmt_snippet)
    composer.add_snippet(container)

    print("Generate header file: {}".format(header_fname))
    with open(header_fname, "w") as wf:
      wf.write('// Auto generated by utensor-cli\n\n')
      wf.write(header_snippet.render())
    print("Generate source file: {}".format(src_fname))
    with open(src_fname, "w") as wf:
      wf.write('// Auto generated by utensor-cli\n\n')
      wf.write(composer.compose())

  def _prepare_tensor_name(self, tensor_name):
    prepared = tensor_name.replace(":", "_").replace("/", "_")
    return prepared

  def _save_data(self, path, value, tf_dtype):
    if tf_dtype in [tf.uint8, tf.qint8, tf.quint8]:
      np_dtype = np.uint8
    elif tf_dtype in [tf.int32, tf.qint32]:
      np_dtype = np.int32
    else:
      np_dtype = np.float32

    if value.shape == ():
      value = np.array([value], dtype=np_dtype)
    else:
      value = value.astype(np_dtype)
    with open(path, "wb") as fid:
      idx2np.convert_to_file(fid, value)
    print("saving {}".format(path))

==> utensor_cgen-0.1.10.tar.gz: utensor_cgen-0.1.10/utensor_cgen/code_generator.py <==
# -*- coding:utf8 -*-
import os
from tempfile import NamedTemporaryFile

import idx2numpy as idx2np
import numpy as np
import tensorflow as tf
from tensorflow.core.framework.graph_pb2 import GraphDef
from tensorflow.tools.graph_transforms import TransformGraph

from .operators import OperatorFactory
from .transformer import NamescopeTransformer
from .optimizer import RefCntOptimizer
from .parser import parse_pb
from .snippets import (CommentSnippet, ContextHeaderSnippet,
                       ContextSnippetsContainer, CreateTensorIdxSnippet)
from .snippets.composer import Composer

__all__ = ["CodeGenerator"]


class CodeGenerator(object):
  def __init__(self, pb_file, idx_dir, embed_data_dir, method, output_nodes, 
               debug_cmt=False,
               clear_tmp_file=True):
    self.pb_file = pb_file
    if not os.path.exists(idx_dir):
      os.makedirs(idx_dir)
    self.idx_dir = idx_dir
    self.embed_data_dir = embed_data_dir.rstrip("/")
    self.method = method
    self.output_nodes = output_nodes
    self.debug_cmt = debug_cmt
    self.clear_tmp_file = clear_tmp_file

  def generate(self, src_fname):
    """Generate source and header files
    """
    fname, _ = os.path.splitext(src_fname)
    graph_name, _ = os.path.splitext(os.path.basename(self.pb_file))
    guard_name = fname.replace('/', '_')
    header_snippet = ContextHeaderSnippet(guard_name, graph_name)

    composer = Composer()
    header_fname = '{}.hpp'.format(fname)
    header_name = os.path.basename(header_fname)
    container = ContextSnippetsContainer(graph_name, header_name)

    opFactory = OperatorFactory()

    print("Transforming graph: {}".format(self.pb_file))
    new_pb_file = self._transform_graph(self.pb_file)
    print("Parsing new pb file {}".format(new_pb_file))
    ops_info, topo_orders = parse_pb(new_pb_file, self.output_nodes)
    if self.clear_tmp_file:
      os.remove(new_pb_file)
    construct_order = RefCntOptimizer.optimize(ops_info, topo_orders, self.output_nodes, self.method)

    # TODO better snippet construction abstraction
    for op_id, (op_name, op_info, ref_counts, to_eval) in enumerate(construct_order, 1):
      op_type = op_info.op_type
      if op_type == "Placeholder":
        out_tname = op_info.output_tensor[0].name
        ref_count = ref_counts[0]
        container.template_vars["placeholders"].append(out_tname)
        container.template_vars["ref_counts"].append(ref_count)
        header_snippet.template_vars["placeholders"].append(out_tname)
      elif op_type == 'Const':
        out_tname, out_dtype, _ = op_info.output_tensor[0]
        ref_count = ref_counts[0]
        pre_tname = self._prepare_tensor_name(out_tname)
        idx_fname = "{}.idx".format(pre_tname)
        snippet = CreateTensorIdxSnippet(self.embed_data_dir, out_tname,
                                         idx_fname=idx_fname,
                                         tf_dtype=out_dtype,
                                         ref_count=ref_count)
        container.add_snippet(snippet)
        idx_path = os.path.join(self.idx_dir, idx_fname)
        value = op_info.output_content[out_tname]
        self._save_data(idx_path, value, out_dtype)
      else:
        snippet = opFactory.createOperatorSnippet(op_info, ref_counts, to_eval)
        container.add_snippet(snippet)

      if self.debug_cmt:
        comments = ["<<< Operation id {}: {}".format(op_id, op_name),
                    ">>> Operation id {}: {}".format(op_id + 1, op_name)]
        cmt_snippet = CommentSnippet(comments)
        container.add_snippet(cmt_snippet)
    composer.add_snippet(container)

    print("Generate header file: {}".format(header_fname))
    with open(header_fname, "w") as wf:
      wf.write('// Auto generated by utensor-cli\n\n')
      wf.write(header_snippet.render())
    print("Generate source file: {}".format(src_fname))
    with open(src_fname, "w") as wf:
      wf.write('// Auto generated by utensor-cli\n\n')
      wf.write(composer.compose())

  def _transform_graph(self, pb_file):
    with tf.gfile.FastGFile(pb_file, 'rb') as fid:
      graph_def = GraphDef()
      graph_def.ParseFromString(fid.read())
    with NamedTemporaryFile(mode='wb',
                            prefix='uTensor_',
                            delete=False,
                            suffix='.pb') as temp_file:
      dropout_transformer = NamescopeTransformer('dropout')
      new_graph_def = dropout_transformer.transform(graph_def, self.output_nodes)
      quant_graph_def = TransformGraph(input_graph_def=new_graph_def,
                                       inputs=[],
                                       outputs=self.output_nodes,
                                       transforms=["quantize_weights", "quantize_nodes"])
      temp_file.write(quant_graph_def.SerializeToString())
    return temp_file.name

  def _prepare_tensor_name(self, tensor_name):
    prepared = tensor_name.replace(":", "_").replace("/", "_")
    return prepared

  def _save_data(self, path, value, tf_dtype):
    if tf_dtype in [tf.uint8, tf.qint8, tf.quint8]:
      np_dtype = np.uint8
    elif tf_dtype in [tf.int32, tf.qint32]:
      np_dtype = np.int32
    else:
      np_dtype = np.float32

    if value.shape == ():
      value = np.array([value], dtype=np_dtype)
    else:
      value = value.astype(np_dtype)
    with open(path, "wb") as fid:
      idx2np.convert_to_file(fid, value)
    print("saving {}".format(path))
